@@ -14,6 +14,7 @@ import shapely
 from shapely.geometry import Polygon as PolygonS, Point as PointS
 from MJOLNIR import TasUBlibDEG as TasUBlib
 from MJOLNIR._tools import Marray
+from MJOLNIR.Data import Mask
 import MJOLNIR.Data.Sample
 import re
 import copy
@@ -51,7 +52,8 @@ class DataFile(object):
             self.name = os.path.basename(fileLocation)
             self.fileLocation = os.path.abspath(fileLocation)		
             self._binning = 1
-			
+            self._mask = False
+
             if not self.type == 'MultiFLEXX':
                 with hdf.File(fileLocation,mode='a') as f:
                     sample=f.get('/entry/sample')
@@ -81,8 +83,8 @@ class DataFile(object):
                     else:
                         self.Monitor=np.array(f.get('entry/data/monitor'))
                     self.Time = np.array(f.get('entry/control/time'))
-                    self.startTime = np.array(f.get('entry/start_time'))
-                    self.endTime = np.array(f.get('entry/end_time'))
+                    self.startTime = np.array(f.get('entry/start_time'))[0]
+                    self.endTime = np.array(f.get('entry/end_time'))[0]
                     self.experimentIdentifier = np.array(f.get('entry/experiment_identifier'))[0]
                     self.comment = np.array(f.get('entry/comment'))[0]
                     self.proposalId = np.array(f.get('entry/proposal_id'))[0]
@@ -188,6 +190,7 @@ class DataFile(object):
             else: # type is multiFLEXX
                 self.loadMultiFLEXXData(fileLocation)
             
+            self.scanSteps = self.scanValues.shape[1]
             if True:
                 for attr in dir(self):
                     # If attribute is a function or property, skip it
@@ -315,6 +318,34 @@ class DataFile(object):
             
             self._binning = value
 
+    @property
+    def mask(self):
+        return self._mask
+
+    @mask.getter
+    def mask(self):
+        return self._mask
+
+    @mask.setter
+    def mask(self,mask):
+        if isinstance(mask,Mask.MaskingObject):
+            coordsInside = np.array([hasattr(self,coord) for coord in mask.coordinates])
+            if not np.all(coordsInside):
+                names = np.array(mask.coordinates)
+                raise AttributeError('Provided mask has coord(s) not in DataFile:','\n'.join(names[np.logical_not(coordsInside)]))
+            self._maskingObject = mask
+        elif hasattr(self,'_maskingObject'): # if _maskingObject exists from ealier, delete it
+            del self._maskingObject
+        if hasattr(self,'I'): # if identity has    
+            if hasattr(self,'_maskingObject'):
+                self._mask = self._maskingObject(self) # Generate boolean mask
+            else:
+                if not np.all(self.I.shape == mask.shape):
+                    raise AttributeError('Shape of provided mask {} does not match shape of data {}.'.format(mask.shape,self.I.shape))
+                self._mask = mask
+        else:
+            self._mask = mask
+
     def updateProperty(self,dictionary):
         if isinstance(dictionary,dict):
             for key in dictionary.keys():
@@ -664,7 +695,7 @@ class DataFile(object):
         if self.type in ['MultiFLEXX','FlatCone']:
             Data.shape = (Data.shape[0],Data.shape[1],-1)
 
-        A4Zero = self.A4Off.copy()#file.get('entry/sample/polar_angle_zero')
+        A4Zero = self.A4Off#file.get('entry/sample/polar_angle_zero')
         
         
         if A4Zero is None:
@@ -1180,10 +1211,10 @@ class DataFile(object):
         
         def addMetaData(self,entry):
             dset = entry.create_dataset('start_time',(1,),dtype='<S70')
-            dset[0] = self.startTime
+            dset[0] = np.string_(self.startTime)
 
             dset = entry.create_dataset('end_time',(1,),dtype='<S70')
-            dset[0] = self.endTime
+            dset[0] = np.string_(self.endTime)
             
             dset = entry.create_dataset('experiment_identifier',(1,),dtype='<S70')
             dset[0] = self.experimentIdentifier.encode('utf8')
@@ -1667,7 +1698,7 @@ def extractData(files):
     mask = []
     for datafile in files:
         I.append(datafile.I)
-        if(files[0].type=='nxs'):
+        if(datafile.type=='nxs'):
             qx.append(datafile.qx)
             qy.append(datafile.qy)
             energy.append(datafile.energy)
